// Bentigt GLUT!
// Fr Dev-C++ (also Mingw32) siehe http://sites.netscape.net/ptrpck/glut.htm
// mit GLUT linken: Project/Project options/Further object files or linker options:
// -lopengl32 -lglu32 -lglut32

// Korrekturen fr GLUT in Dev-C++:
#define GLUT_NO_LIB_PRAGMA
#define GLUT_NO_WARNING_DISABLE

#include <gl/glut.h>
#include "vector3d.h"
#include <vector>
#include <algorithm>
#include <math.h>

std::vector<Vector3D> vertexList1;
std::vector<Vector3D> vertexList2;
std::vector<double> distanceList;

Vector3D f(double t, Vector3D u) // Lorenz-System; t unbenutzt
{
    const double a = 10.0;
    const double r = 28.0;
    const double b = 8.0/3.0;
    return Vector3D(a*(u.y-u.x), r*u.x-u.y-u.x*u.z, u.x*u.y-b*u.z);
}

// Runge-Kutta-Verfahren, u = Startvektor, Ergebnis nach vertexList
void solveRungeKutta(std::vector<Vector3D>* vertexList, Vector3D u)
{
    const double dt = 4.0 * 1.0e-3;
    double t = 0.0;
    while(t < 50.0)
    {
        vertexList->push_back(u);

        Vector3D du1 = dt*f(t, u);
        t += 0.5*dt;
        Vector3D du2 = dt*f(t, u+0.5*du1);
        Vector3D du3 = dt*f(t, u+0.5*du2);
        t += 0.5*dt;
        Vector3D du4 = dt*f(t, u+du3);
        u = u + (1.0/6.0)*du1 + (1.0/3.0)*du2 + (1.0/3.0)*du3 + (1.0/6.0)*du4;
    }
}

void prepareData()
{
    solveRungeKutta(&vertexList1, Vector3D(2.0, 2.0, 1.0));
    solveRungeKutta(&vertexList2, Vector3D(2.0, 2.001, 1.0));
    unsigned int i;
    for(i = 0; i < vertexList1.size(); i++)
    {
        distanceList.push_back( (vertexList1[i]-vertexList2[i]).length() );
    }
}

void plot(Vector3D v)
{
    v.plot();
}

void renderScene1()
{
    glClear(GL_COLOR_BUFFER_BIT);

    glBegin(GL_LINE_STRIP);
        glColor3f(1.0, 0.0, 1.0);
        std::for_each(vertexList1.begin(), vertexList1.end(), plot);
    glEnd();
    glBegin(GL_LINE_STRIP);
        glColor3f(0.0, 1.0, 0.0);
        std::for_each(vertexList2.begin(), vertexList2.end(), plot);
    glEnd();

    glutSwapBuffers();
}

void changeSize1(GLsizei w, GLsizei h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(60.0, h==0 ? 1.0 : w/(double)h, 1.0, 500.0);

    glTranslatef(0.0, 0.0, -100.0);
}

void renderScene2()
{
    double maxD = 1.0e-100;
    double minLogD = 1.0e100;
    unsigned int i;
    for(i = 0; i < distanceList.size(); i++)
    {
        if( distanceList[i] > maxD)
        {
            maxD = distanceList[i];
        }

        double lg = log(distanceList[i]);
        if( lg < minLogD)
        {
            minLogD = lg;
        }
    }

    glClear(GL_COLOR_BUFFER_BIT);

    glBegin(GL_LINES);
        glColor3f(0.5, 0.5, 0.5);
        glVertex2f(0.0, 1.0);
        glVertex2f(1.0, 1.0);
    glEnd();

    glBegin(GL_LINE_STRIP);
        glColor3f(0.0, 1.0, 1.0);
        for(i = 0; i < distanceList.size(); i++)
        {
            // Kurve in obere Hlfte: y + 1
            glVertex2f(i/(double)distanceList.size(), 1.0+distanceList[i]/maxD);
        }
    glEnd();

    glBegin(GL_LINE_STRIP);
    glColor3f(1.0, 1.0, 0.0);
    for(i = 0; i < distanceList.size(); i++)
    {
        double lg = log(distanceList[i])-minLogD;
        if(lg > 8)
        {
            lg = 8;
        }
        glVertex2f(i/(double)distanceList.size(), lg/8.0);
    }
    glEnd();

    glFlush();
}

void changeSize2(GLsizei w, GLsizei h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0.0, 1.0, 0.0, 2.0, -1.0, 1.0);
}

void specialFunc(int key, int x, int y)
{
    glMatrixMode(GL_MODELVIEW);

    float oldModelView[16];
    glGetFloatv(GL_MODELVIEW_MATRIX, oldModelView);
    glLoadIdentity();

    switch(key)
    {
    case GLUT_KEY_RIGHT:
        glRotatef(2.0, 0.0, 1.0, 0.0);
        break;
    case GLUT_KEY_LEFT:
        glRotatef(-2.0, 0.0, 1.0, 0.0);
        break;
    case GLUT_KEY_UP:
        glRotatef(-2.0, 1.0, 0.0, 0.0);
        break;
    case GLUT_KEY_DOWN:
        glRotatef(2.0, 1.0, 0.0, 0.0);
        break;
    default:
        ;
    }

    glMultMatrixf(oldModelView);

    glutPostRedisplay();
}

int main(int argc, char* argv[])
{
    prepareData();

    glutInit(&argc, argv);

    glutInitWindowSize(650, 650);
    glutInitWindowPosition(45, 45);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
    glutCreateWindow("Lorenz-System");
    glutDisplayFunc(renderScene1);
    glutReshapeFunc(changeSize1);
    glutSpecialFunc(specialFunc);

    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glutInitWindowSize(600, 400);
    glutInitWindowPosition(30, 30);
    glutInitDisplayMode(GLUT_RGB | GLUT_SINGLE);
    glutCreateWindow("Strungswachstum");
    glutDisplayFunc(renderScene2);
    glutReshapeFunc(changeSize2);

    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glutMainLoop();

    return 0; // nie erreicht
}
